/*  This file is part of MED.
 *
 *  COPYRIGHT (C) 1999 - 2011  EDF R&D, CEA/DEN
 *  MED is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  MED is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public License
 *  along with MED.  If not, see <http://www.gnu.org/licenses/>.
 */



#include <med.h>
#include <med_config.h>
#include <med_outils.h>
#include <string.h>
#include <stdlib.h>

#include <2.3.6/med23v30.h>
#include <2.3.6/med23v30_proto.h>
#include "2.3.6/med23v30_misc.h"

void _MEDsubdomainCorrespondenceSizeInfo236(int dummy, ...) {

  va_list params;
  va_start(params,dummy);

  const med_idt              fid            = va_arg(params,const med_idt               );
  const char * const         meshname       = va_arg(params,const char * const          );
  const char * const         jointname      = va_arg(params,const char * const          );
  const med_int              numdt          = va_arg(params,const med_int               );
  const med_int              numit          = va_arg(params,const med_int               );
  const int                  corit          = va_arg(params,const int                   );
  med_entity_type   * const  localentitype  = va_arg(params,med_entity_type   * const   );
  med_geometry_type * const  localgeotype   = va_arg(params,med_geometry_type * const   );
  med_entity_type   * const  remoteentitype = va_arg(params,med_entity_type   * const   );
  med_geometry_type * const  remotegeotype  = va_arg(params,med_geometry_type * const   );
  med_int * const            nentitycor     = va_arg(params,med_int * const             );
  med_err *                  fret           = va_arg(params,med_err *);

  med_err  _ret=-1;
  med_int  _nentitycor=0;

  /*
   * On inhibe le gestionnaire d'erreur
   */
  _MEDmodeErreurVerrouiller();

  if ( (numdt != MED_NO_DT) || (numit != MED_NO_IT) ) {
    MED_ERR_(_ret,MED_ERR_INVALID,MED_ERR_PARAMETER,"");
    ISCRUTE(numdt);ISCRUTE(numit);goto ERROR;
  }


  if ( MEDjointTypeCorres( fid, (char *) meshname, (char *) jointname,corit,
			   (med_entite_maillage * ) localentitype , (med_geometrie_element*) localgeotype,
			   (med_entite_maillage * ) remoteentitype, (med_geometrie_element*) remotegeotype) < 0 ) {
    MED_ERR_(_ret,MED_ERR_CALL,MED_ERR_API,"MEDjointTypeCorres");
    SSCRUTE(meshname);SSCRUTE(jointname);
    goto ERROR;
  }

  if ( (_nentitycor=MEDjointnCorres(fid, (char *) meshname, (char *) jointname,
				    (med_entite_maillage ) *localentitype , (med_geometrie_element) *localgeotype,
				    (med_entite_maillage ) *remoteentitype, (med_geometrie_element) *remotegeotype) ) < 0 ) {
    MED_ERR_(_ret,MED_ERR_CALL,MED_ERR_API,"MEDjointnCorres");
    SSCRUTE(meshname);SSCRUTE(jointname);
    goto ERROR;
  }
  *nentitycor = _nentitycor;
/*   ISCRUTE(*nentitycor); */

  _ret = 0;
 ERROR:

  va_end(params);
  *fret = _ret;
  return;
}
